@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -47,45 +48,45 @@ FFN(Fcatch)(VOL object args)
 	frs_pop();
 }
 
-DEFUN_NEW("ERROR-SET",object,fSerror_set,SI
+DEFCONST("+TOP-ABORT-TAG+",sSPtop_abort_tagP,SI,MMcons(Cnil,Cnil),"");
+
+DEFUNM("ERROR-SET",object,fSerror_set,SI
 	   ,1,1,NONE,OO,OO,OO,OO,(volatile object x0),
        "Evaluates the FORM in the null environment.  If the evaluation \
 of the FORM has successfully completed, SI:ERROR-SET returns NIL as the first \
 value and the result of the evaluation as the rest of the values.  If, in the \
 course of the evaluation, a non-local jump from the FORM is atempted, \
 SI:ERROR-SET traps the jump and returns the corresponding jump tag as its \
-value.")
+value.") {
 
-{
-	object *old_lex = lex_env;
+  object *old_lex = lex_env;
+  object *vals=(object *)fcall.valp;
+  
+  vs_push(Cnil);
+  frs_push(FRS_CATCHALL, Cnil);
+
+  if (nlj_active) {
+
+    nlj_active = FALSE;
+    x0 = nlj_tag;
+    frs_pop();
+    lex_env = old_lex;
+    RETURN1(x0);
+
+  }
+
+  lex_env = vs_top;
+  vs_push(Cnil);
+  vs_push(Cnil);
+  vs_push(Cnil);
+  if (vals)
+    vals[0]=Ievaln(x0,vals+1);
+  else
+    Ieval1(x0);
+  frs_pop();
+  lex_env = old_lex;
+  RETURN1(Cnil);
 
-	/* 1 args */
-	vs_push(Cnil);
-	frs_push(FRS_CATCHALL, Cnil);
-	if (nlj_active) {
-		nlj_active = FALSE;
-		x0 = nlj_tag;
-		frs_pop();
-		lex_env = old_lex;
-		RETURN1(x0);
-	} else {
-		lex_env = vs_top;
-		vs_push(Cnil);
-		vs_push(Cnil);
-		vs_push(Cnil);
-		x0 = Ieval(x0);
-	}
-	frs_pop();
-	lex_env = old_lex;
-	{int i = fcall.nvalues;
-	if (i+1>=sizeof(fcall.values)/sizeof(*fcall.values))
-	  FEerror("Too many function call values",0);
-	while (i > 0)
-	{ fcall.values[i+1] = fcall.values[i];
-	  i--;}
-	 fcall.nvalues++;
-	 fcall.values[1] = x0;}
-	return Cnil;
 }
 
 static void
@@ -150,7 +151,8 @@ FFN(Fthrow)(object args)
 	vs_push(tag);
 	fr = frs_sch_catch(tag);
 	if (fr == NULL)
-		FEerror("~S is an undefined tag.", 1, tag);
+/* 		FEerror("~S is an undefined tag.", 1, tag); */
+	  CONTROL_ERROR("Undefined tag.");
 	eval(MMcadr(args));
 	unwind(fr, tag);
 	/* never reached */
