@@ -1,4 +1,5 @@
  /* Copyright William F. Schelter  All Rights Reserved.
+    Copyright 2024 Camm Maguire
 
    Utility for writing out lisp objects and reading them in:
    Basically it attempts to write out only those things which could
@@ -100,14 +101,14 @@ enum dump_type {
 
 /* set whole structures!  */
 #define SETUP_FASD_IN(fd) do{ \
-  fas_stream= (fd)->stream->sm.sm_fp; \
+  fas_stream= (fd)->stream; \
   dump_index =   fix((fd)->index) ; \
   current_fasd= * (fd);}while(0)
 
 #define SAVE_CURRENT_FASD \
    struct fasd old_fd; \
    int old_dump_index = dump_index; \
-   FILE *old_fas_stream = fas_stream; \
+   object old_fas_stream = fas_stream; \
    int old_needs_patching = needs_patching; \
    old_fd = current_fasd;
 
@@ -137,9 +138,9 @@ enum dump_type {
 
 #define FASD_VERSION 2
 
-FILE *fas_stream;
+object fas_stream;
 int dump_index;
-struct htent *gethash();
+/* struct htent *gethash(); */
 static void read_fasd1(int i, object *loc);
 object extended_read();
 
@@ -148,10 +149,10 @@ object extended_read();
 */   
 /* #define DEBUG */
 
-#ifdef DEBUG
+#ifdef DEBUG /*FIXME debugging versions need sync with getc -> readc_stream, etc.*/
 
-#define PUT(x) putc1((char)x,fas_stream)
-#define GET() getc1()
+#define PUT(x) writec_stream1((char)x,fas_stream)
+#define GET() readc_stream1()
 #define D_FWRITE fwrite1
 #define D_FREAD fread1
 
@@ -220,22 +221,22 @@ char *str;
    {printf("{");
     printf(str,i);
     printf("}");}
- putc(i,fas_stream);}
+ writec_stream(i,fas_stream);}
 
 void
-putc1(x)
+writec_stream1(x)
 int x;
 {  if (debug) printf("(%x,%d,%c)",x,x,x);
-   putc(x,fas_stream);
-   fflush(stdout);
+   writec_stream(x,fas_stream);
+/*    fflush(stdout); */
  }
 
 int
-getc1()
+readc_stream1()
 { int x;
-   x= getc(fas_stream);
+   x= readc_stream(fas_stream);
   if (debug) printf("(%x,%d,%c)",x,x,x);
-  fflush(stdout);
+/*   fflush(stdout); */
   return x;
  }
 
@@ -251,9 +252,9 @@ fread1(p,n1,n2,st)
  {printf("[");
   n1=n1*n2;
   for(i=0;i<n1; i++)
-    putc(p[i],stdout);
+    writec_stream(p[i],sLAstandard_outputA->s.s_dbind);
   printf("]");
-  fflush(stdout);}
+/*   fflush(stdout);} */
     return j;
 
 }
@@ -273,14 +274,14 @@ fwrite1(p,n1,n2,st)
  {printf("[");
   n1=n1*n2;
   for(i=0;i<n1; i++)
-    putc(p[i],stdout);
+    writec_stream(p[i],sLAstandard_outputA->s.s_dbind);
   printf("]");}
     return j;
 }
 
 
-#define GET_OP() (print_op(getc(fas_stream)))
-#define PUT_OP(x) fputc(print_op(x),fas_stream)
+#define GET_OP() ((unsigned)print_op((unsigned char)readc_stream(fas_stream)))
+#define PUT_OP(x) writec_stream(print_op(x),fas_stream)
  
 #define DP(sw)  sw   /*  if (debug) {printf("\ncase sw");} */
 #define GETD(str) getd(str)
@@ -288,7 +289,7 @@ fwrite1(p,n1,n2,st)
 int
 getd(str)
  char *str;
-{ int i = getc(fas_stream);
+{ int i = (unsigned char)readc_stream(fas_stream);
  if(debug){
    printf("{");
    printf(str,i);
@@ -296,16 +297,21 @@ getd(str)
   return i;}
 #define DPRINTF(a,b)  do{if(debug) printf(a,b);} while(0)
 #else
-#define PUT(x) putc((char)x,fas_stream)
-#define GET() getc(fas_stream)
+#define PUT(x) writec_stream((char)x,fas_stream)
+#define GET() ((unsigned char)readc_stream(fas_stream))
 #define GET_OP GET
 #define PUT_OP PUT
-#define D_FWRITE fwrite
-#define D_FREAD SAFE_FREAD
+#define D_FWRITE fwrite_int
+#define D_FREAD fread_int
 #define DP(sw)  sw
 #define PUTD(a,b) PUT(b)
 #define GETD(a) GET()
 #define DPRINTF(a,b)  
+/* #define fwrite_int(a_,b_,c_,d_) {register char *_p=(a_),*_pe=_p+(b_)*(c_);for (;_p<_pe;) writec_stream(*_p++,(d_));} */
+/* #define fread_int(a_,b_,c_,d_)  {register char *_p=(a_),*_pe=_p+(b_)*(c_);for (;_p<_pe;) *_p++=readc_stream(d_);} */
+#define fwrite_int(a_,b_,c_,d_) {register unsigned _i;for (_i=0;_i<(b_)*(c_);_i++) writec_stream(((char *)(a_))[_i],(d_));}
+#define fread_int(a_,b_,c_,d_)  {register unsigned _i;for (_i=0;_i<(b_)*(c_);_i++) ((char *)(a_))[_i]=readc_stream(d_);}
+
 
 #endif
 
@@ -326,7 +332,7 @@ getd(str)
 
 #define MAKE_SHORT(top,bot) (((top)<< SIZE_BYTE) + (bot))
 
-#define READ_BYTE1() getc(fas_stream)
+#define READ_BYTE1() ((unsigned char)readc_stream(fas_stream))
 
 #define GET8(varx ) \
  do{unsigned long long var=READ_BYTE1();  \
@@ -364,7 +370,7 @@ getd(str)
 
 
 #define MASK ~(~0UL << 8)
-#define WRITE_BYTEI(x,i)  putc((((x) >> (i*SIZE_BYTE)) & MASK),fas_stream)
+#define WRITE_BYTEI(x,i)  writec_stream((((x) >> (i*SIZE_BYTE)) & MASK),fas_stream)
 
 #define PUTFIX(v_) Join(PUT,SIZEOF_LONG)(v_)
 #define GETFIX(v_) Join(GET,SIZEOF_LONG)(v_)
@@ -426,7 +432,6 @@ getd(str)
 #define READ_SYMBOL(leng,pack,to) \
 	do { BEGIN_NO_INTERRUPT;{char  *p=alloc_relblock(leng);\
 	 D_FREAD(p,1,leng,fas_stream); \
-	 string_register->st.st_fillp = \
 	 string_register->st.st_dim = leng; \
 	 string_register->st.st_self = p; \
 	 to=(pack==Cnil ? make_symbol(string_register) : intern(string_register,pack)); \
@@ -449,10 +454,10 @@ getd(str)
 
 static enum circ_ind
 do_hash(object obj, int dot)
-{    struct htent *e;
+{    struct cons *e;
      int i;
      e=gethash(obj,dump_hash_table); 
-     if (e->hte_key==OBJNULL) 
+     if (e->c_cdr==OBJNULL) 
 /* We won't index things unless they have  < -2 in the hash table */
   {   if(type_of(obj)!=t_package) return NOT_INDEXED;
       sethash(obj,dump_hash_table,make_fixnum(dump_index));
@@ -462,12 +467,12 @@ do_hash(object obj, int dot)
 	dump_index++;
 	return FIRST_INDEX;}
 	
-     i = fix(e->hte_value);
+     i = fix(e->c_car);
      if (i == -1) return NOT_INDEXED; /* don't want to index this baby */
      
      if (dot) PUT_OP(dot);
      if ( i < -1)
-       { e->hte_value = make_fixnum(dump_index);
+       { e->c_car = make_fixnum(dump_index);
 	 PUT_OP(d_new_indexed_item);
 	 DPRINTF("{dump_index=%d}",dump_index);
 	 dump_index++;
@@ -496,10 +501,14 @@ do_hash(object obj, int dot)
    }
  
 static void write_fasd(object obj);
-DEFUN_NEW("WRITE-FASD-TOP",object,fSwrite_fasd_top,SI,2,2,NONE,OO,OO,OO,OO,(object obj, object x),"")
+
+DEFUN("WRITE-FASD-TOP",object,fSwrite_fasd_top,SI
+	  ,2,2,NONE,OO,OO,OO,OO,(object obj, object x),"") {
+
 /* static object */
 /* FFN(write_fasd_top)(object obj, object x) */
-{struct fasd *fd = (struct fasd *) x->v.v_self;
+/* { */
+struct fasd *fd = (struct fasd *) x->v.v_self;
   if (fd->direction == sKoutput)
     SETUP_FASD_IN(fd);
   else FEerror("bad value for open slot of fasd",0);
@@ -519,10 +528,12 @@ DEFUN_NEW("WRITE-FASD-TOP",object,fSwrite_fasd_top,SI,2,2,NONE,OO,OO,OO,OO,(obje
 #define MAYBE_PATCH(result) \
   if (needs_patching)  result =fasd_patch_sharp(result,0)
 
-DEFUN_NEW("READ-FASD-TOP",object,fSread_fasd_top,SI,1,1,NONE,OO,OO,OO,OO,(object x),"")
+DEFUN("READ-FASD-TOP",object,fSread_fasd_top,SI
+	  ,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 /* static object */
 /* FFN(read_fasd_top)(object x) */
-{  struct fasd *fd = (struct fasd *)  x->v.v_self;
+/* { */
+  struct fasd *fd = (struct fasd *)  x->v.v_self;
    VOL int e=0;
    object result;
    SAVE_CURRENT_FASD;
@@ -573,57 +584,61 @@ object sLeq;
 object sSPinit;
 void Lmake_hash_table();
 
-DEFUN_NEW("OPEN-FASD",object,fSopen_fasd,SI,4,4,NONE,OO,OO,OO,OO,(object stream, object direction, object eof, object tabl),"")
+DEFUN("OPEN-FASD",object,fSopen_fasd,SI
+	  ,4,4,NONE,OO,OO,OO,OO,(object stream, object direction, object eof, object tabl),"") {
+
+
 /* static object */
 /* FFN(open_fasd)(object stream, object direction, object eof, object tabl) */
-{  object str=Cnil;
-   object result;
-   if(direction==sKinput)
-     {str=coerce_stream(stream,0);
-      if (tabl==Cnil)
-	tabl=alloc_simple_vector(0,aet_object);
-      else
-	check_type(tabl,t_vector);}
-   if(direction==sKoutput)
-     {str=coerce_stream(stream,1);
-       if(tabl==Cnil) tabl=gcl_make_hash_table(sLeq);
-      else
-	check_type(tabl,t_hashtable);}
-   massert(str==stream);
-   result=alloc_simple_vector(sizeof(struct fasd)/sizeof(object),aet_object);
-   array_allocself(result,1,Cnil);
-   {struct fasd *fd= (struct fasd *)result->v.v_self;
-    fd->table=tabl;
-    fd->stream=stream;
-    fd->direction=direction;
-    fd->eof=eof;
-    fd->index=small_fixnum(0);
-    fd->package=symbol_value(sLApackageA);
-    fd->filepos = make_fixnum(ftell(stream->sm.sm_fp));
+/* { */
+  object str=Cnil;
+  object result;
+  if(direction==sKinput)
+    {str=coerce_stream(stream,0);
+    if (tabl==Cnil)
+      tabl=alloc_simple_vector(0);
+    else
+      check_type(tabl,t_simple_vector);}
+  if(direction==sKoutput)
+    {str=coerce_stream(stream,1);
+      if(tabl==Cnil) tabl=gcl_make_hash_table(sLeq);
+    else
+      check_type(tabl,t_hashtable);}
+  massert(str==stream);
+  result=alloc_simple_vector(sizeof(struct fasd)/sizeof(object));
+  array_allocself(result,1,Cnil);
+  {struct fasd *fd= (struct fasd *)result->sv.sv_self;
+  fd->table=tabl;
+  fd->stream=stream;
+  fd->direction=direction;
+  fd->eof=eof;
+  fd->index=small_fixnum(0);
+  fd->package=symbol_value(sLApackageA);
+  fd->filepos = make_fixnum(file_position(stream));
+  
+  SETUP_FASD_IN(fd);
+  if (direction==sKoutput){
+    PUT_OP((int)d_begin_dump);
+    PUTD("version=%d",FASD_VERSION);
+    PUT4(0);  /* reserve space for the size of index array needed */
+    /*  equivalent to:   write_fasd(current_fasd.package);
+	except we don't want to index this, so that we can open
+	with an empty array.
+    */
+    PUT_OP(d_package);
+    write_fasd(current_fasd.package->p.p_name);
     
-    SETUP_FASD_IN(fd);
-    if (direction==sKoutput){
-      PUT_OP((int)d_begin_dump);
-      PUTD("version=%d",FASD_VERSION);
-      PUT4(0);  /* reserve space for the size of index array needed */
-          /*  equivalent to:   write_fasd(current_fasd.package);
-	      except we don't want to index this, so that we can open
-	      with an empty array.
-	   */
-      PUT_OP(d_package);
-      write_fasd(current_fasd.package->p.p_name);
-
+  }
+  else			/* input */
+    { object tem;
+    read_fasd1(GET_OP(),&tem);
+    if(tem!=current_fasd.table) FEerror("not positioned at beginning of a dump",0);
     }
-    else			/* input */
-      { object tem;
-	read_fasd1(GET_OP(),&tem);
-	if(tem!=current_fasd.table) FEerror("not positioned at beginning of a dump",0);
-      }
-    fd->index=make_fixnum(dump_index);
-    fd->filepos=current_fasd.filepos;
-    fd->package=current_fasd.package;
-    fd->table_length=current_fasd.table_length;
-    return result;
+  fd->index=make_fixnum(dump_index);
+  fd->filepos=current_fasd.filepos;
+  fd->package=current_fasd.package;
+  fd->table_length=current_fasd.table_length;
+  return result;
   }}
 #ifdef STATIC_FUNCTION_POINTERS
 object
@@ -632,28 +647,23 @@ fSopen_fasd(object stream, object direction, object eof, object tabl) {
 }
 #endif
 
-DEFUN_NEW("CLOSE-FASD",object,fSclose_fasd,SI,1,1,NONE,OO,OO,OO,OO,(object ar),"")
+DEFUN("CLOSE-FASD",object,fSclose_fasd,SI,1,1,NONE,OO,OO,OO,OO,(object ar),"") {
 /* static object */
 /* FFN(close_fasd)(object ar) */
-{  struct fasd *fd= (struct fasd *)(ar->v.v_self);
-   check_type(ar,t_vector);
-   if (type_of(fd->table)==t_vector)
-     /* input uses a vector */
-     {if (fd->table->v.v_self)
-	 gset(fd->table->v.v_self,0,fix(fd->index),aet_object);
-    }
-   else
+/* { */
+  struct fasd *fd= (struct fasd *)(ar->v.v_self);
+   check_type(ar,t_simple_vector);
      if(fd->direction==sKoutput)
        {clrhash(fd->table);
 	SETUP_FASD_IN(fd);
 	PUT_OP(d_end_of_file);
-	{int i = ftell(fd->stream->sm.sm_fp);
+	{int i = file_position(fd->stream);
 	 if(type_of(fd->filepos) == t_fixnum)
-	   { fseek(fd->stream->sm.sm_fp,fix(fd->filepos)+2,SEEK_SET);
+	  { file_position_set(fd->stream,fix(fd->filepos) +2);
 	    /* record the length of array needed to read the indices */
 	    PUT4(fix(fd->index));
 	    /* move back to where we were */
-	    fseek(fd->stream->sm.sm_fp,i,SEEK_SET);
+	    file_position_set(fd->stream,i);
 	  }}
 	 
       }
@@ -705,7 +715,7 @@ write_fasd(object obj)
 	  { if(x==Cnil)
 	      {PUT_OP(d_list1+l);
 	       break;}
-	    if(type_of(x)==t_cons)
+	    if(consp(x))
 	      {if ((int) d_list1 + ++l > (int) d_list4)
 	       {PUT_OP(d_list);
 		break;}
@@ -728,7 +738,7 @@ write_fasd(object obj)
        obj=obj->c.c_cdr;
        {int l=0;
 	while(1)
-	  {if (type_of(obj)==t_cons)
+	  {if (consp(obj))
 	     { enum circ_ind is_indexed=LATER_INDEX;
 	       if(HASHP(t_cons)){
 		 is_indexed=do_hash(obj,d_dot);
@@ -757,7 +767,7 @@ write_fasd(object obj)
        if (obj==Cnil)
 	 {PUT_OP(d_nil); return;}
         TRY_HASH;
-       leng=obj->s.s_fillp;
+	leng=VLEN(obj->s.s_name);
        if (current_fasd.package!=obj->s.s_hpack)
 	 {{
 	   if (leng< SHORT_MAX)
@@ -778,7 +788,7 @@ write_fasd(object obj)
 	     PUT_OP(d_symbol);
 	     PUT4(j);}
 	   }
-       D_FWRITE(obj->s.s_self,1,leng,fas_stream);
+       D_FWRITE(obj->s.s_name->st.st_self,1,leng,fas_stream);
        break;
      case DP(t_fixnum:)
        leng=fix(obj);
@@ -795,8 +805,9 @@ write_fasd(object obj)
        PUT_OP(d_standard_character);
        PUTD("char=%c",char_code(obj));
        break;
+     case DP(t_simple_string:)
      case DP(t_string:)
-       leng=(obj)->st.st_fillp;
+       leng=VLEN(obj);
        if (leng< SHORT_MAX)
 	 {PUT_OP(d_short_string);
 	  PUTD("leng=%d",leng);}
@@ -873,10 +884,11 @@ write_fasd(object obj)
 	    write_fasd(aref(obj,i));}
       break;
 	
+      case DP(t_simple_vector:)
       case DP(t_vector:)
 	TRY_HASH;
 	PUT_OP(d_vector);
-	{ int leng=obj->v.v_fillp;
+	{ int leng=VLEN(obj);
 	  PUT4 (leng);
 	  PUTD("eltype=%d",obj->v.v_elttype);
 	  {int i;
@@ -899,7 +911,7 @@ fasd_patch_sharp_cons(object x, int depth)
 {
 	for (;;) {
 		x->c.c_car = fasd_patch_sharp(x->c.c_car,depth+1);
-		if (type_of(x->c.c_cdr) == t_cons)
+		if (consp(x->c.c_cdr))
 			x = x->c.c_cdr;
 		else {
                         x->c.c_cdr = SAFE_CDR(fasd_patch_sharp(x->c.c_cdr,depth+1));
@@ -938,6 +950,7 @@ fasd_patch_sharp(object x, int depth)
 		fasd_patch_sharp_cons(x,depth);
 		break;
 
+	case DP(t_simple_vector:)
 	case DP(t_vector:)
 	{
 		int i;
@@ -945,7 +958,7 @@ fasd_patch_sharp(object x, int depth)
 		if ((enum aelttype)x->v.v_elttype != aet_object)
 		  break;
 
-		for (i = 0;  i < x->v.v_fillp;  i++)
+		for (i = 0;  i < VLEN(x);  i++)
 			x->v.v_self[i] = fasd_patch_sharp(x->v.v_self[i],depth);
 		break;
 	}
@@ -981,49 +994,38 @@ fasd_patch_sharp(object x, int depth)
 
 object sharing_table;
 
-DEFUN_NEW("FIND-SHARING-TOP",object,fSfind_sharing_top,SI,2,2,NONE,OO,OO,OO,OO,(object x, object table),"") {
+DEFUN("FIND-SHARING-TOP",object,fSfind_sharing_top,SI
+	  ,2,2,NONE,OO,OO,OO,OO,(object x,object table),"") {
+
   sharing_table=table;
   travel_find_sharing(x,table);
-  return Ct;
+  RETURN1(Ct);
+
 }
 
-/* static object            */
-/* read_fasd(int i) */
-/* {object tem; */
-/*    read_fasd1(i,&tem); */
-/*    return tem;} */
+static object
+lisp_eval(object x) {
 
+  SAVE_CURRENT_FASD;
+  x=ieval(x);
+  RESTORE_FASD;
 
-     /* I am not sure if saving vs_top,vs_base is necessary */
-static object 
-lisp_eval(object x)
-{  object *b,*t;
-   SAVE_CURRENT_FASD;
-   b=vs_base;
-   t=vs_top;
-   vs_base=vs_top;
-   vs_push(x);
-   Leval(); 
-   x=vs_base[0];
-   vs_base=b;
-   vs_top=t;
-   RESTORE_FASD;
-   return x;
- }
+  return x;
 
-    
+}
 
-#define CHECK_CH(i)    	   do{if ((i)==EOF && feof(fas_stream)) bad_eof();}while (0)
+#define CHECK_CH(i)    	   do{if ((i)==EOF && stream_at_end(fas_stream)) bad_eof();}while (0)
 /* grow vector AR of general type */
 static void
 grow_vector(object ar)
 {   int len=ar->v.v_dim;
-    int nl=(int) (1.5*len);
+    int nl=(int) (1.5*(len+1));
     {BEGIN_NO_INTERRUPT;
      {char *p= (char *)AR_ALLOC(alloc_contblock,nl,object);
     bcopy(ar->v.v_self,p,sizeof(object)* len);
     ar->v.v_self= (object *)p;
-    ar->v.v_dim=	   ar->v.v_fillp=nl;
+    ar->v.v_dim=nl;
+    VSET_MAX_FILLP(ar);
     while(--nl >=len)
       ar->v.v_self[nl]=Cnil;
     END_NO_INTERRUPT;}}
@@ -1189,7 +1191,7 @@ read_fasd1(int i, object *loc)
       case DP(d_fixnum:)
 	{fixnum j;
 	 GETFIX(j);
-	 *loc=SAFE_CDR(make_fixnum(j));       
+	 *loc=SAFE_CDR(make_fixnum(j));
 	 return;}
       case DP( d_bignum:)
 	{int j,m;
@@ -1242,7 +1244,8 @@ read_fasd1(int i, object *loc)
 	      read_fasd1(GET_OP(),p++);
 	    vs_base=base;
 	    vs_top = p;
-	    siLmake_structure();
+	    funcall(find_symbol(str("MAKE-STRUCTURE"),system_package));
+	    /* siLmake_structure(); */
 	    *loc = vs_base[0];
 	    vs_top=vs_base=base;
 	    return;
@@ -1273,13 +1276,12 @@ read_fasd1(int i, object *loc)
       case DP(d_vector:)
 	{int leng,j;
 	 object y;
-	 object x=alloc_object(t_vector);
+	 object x;
 	 GET4(leng);
-	 x->v.v_elttype = GETD("v_elttype=%d");
-	 x->v.v_dim=x->v.v_fillp=leng;
-	 x->v.v_self=0;
-	 x->v.v_displaced=Cnil;
-	 x->v.v_hasfillp=x->v.v_adjustable=0;
+	 {
+	   enum aelttype tp=GETD("v_elttype=%d");
+	   x= tp==aet_object ? alloc_simple_vector(leng) : alloc_vector(leng,tp);
+	 }
 	 array_allocself(x,0,Cnil);
 	 for (j=0; j< leng ; j++)
 	   { DPRINTF("{vector_elt=%d}",j);
@@ -1298,14 +1300,15 @@ read_fasd1(int i, object *loc)
 	 object x=alloc_object(t_array);
 	 GET4(leng);
 
-	 x->a.a_elttype = GETD("a_elttype=%d");
+	 set_array_elttype(x,GETD("a_elttype=%d"));
 	 x->a.a_dim=leng;
+	 x->a.a_hasfillp=1;
 	 x->a.a_rank= GETD("a_rank=%d");
 	 x->a.a_self=0;
-	 x->a.a_displaced=Cnil;
-	 x->a.a_adjustable=0;
+	 x->a.a_adjustable=1;
+	 SET_ADISP(x,Cnil);
 	 if (x->a.a_rank > 0)
-	   { x->a.a_dims = (int *)alloc_relblock(sizeof(int)*(x->a.a_rank)); }
+	   { x->a.a_dims = (ufixnum *)alloc_relblock(sizeof(fixnum)*(x->a.a_rank)); }
 	 for (i=0; i< x->a.a_rank ; i++)
 	   GET4(x->a.a_dims[i]);
 	 array_allocself(x,0,Cnil);
@@ -1402,8 +1405,8 @@ clrhash(object table)
 {int i;
    if (table->ht.ht_nent > 0 )
      for(i = 0; i < table->ht.ht_size; i++) {
-       table->ht.ht_self[i].hte_key = OBJNULL;
-       table->ht.ht_self[i].hte_value = OBJNULL;}
+       table->ht.ht_self[i].c_cdr = OBJNULL;
+       table->ht.ht_self[i].c_car = OBJNULL;}
    table->ht.ht_nent =0;}
 
 object IfaslInStream;
@@ -1446,6 +1449,6 @@ init_fasdump(void)
 /*   make_si_sfun("WRITE-FASD-TOP",write_fasd_top,2); */
 /*   make_si_sfun("OPEN-FASD",open_fasd,4);   */
 /*   make_si_sfun("CLOSE-FASD",close_fasd,1); */
-/* /\*  make_si_sfun("FASD-I-DATA",fasd_i_macro,1); *\/ */
+/*  make_si_sfun("FASD-I-DATA",fasd_i_macro,1); */
 /*   make_si_sfun("FIND-SHARING-TOP",find_sharing_top,2); */
 }
