@@ -1,4 +1,5 @@
   /* Copyright William F. Schelter 1991
+     Copyright 2024 Camm Maguire
    Bignum routines.
  
 
@@ -43,7 +44,6 @@ read.d: normalize_big_to_object
  */
 
 
-#include <string.h>
 
 #define DEBUG_GMP
 #ifdef DEBUG_GMP
@@ -88,11 +88,23 @@ static object verify_big_or_zero(object big)
 #ifndef GMP_USE_MALLOC
 object big_gcprotect;
 object big_fixnum1;
+object big_fixnum2;
+object big_fixnum3;
+object big_fixnum4;
+object big_fixnum5;
 
 #include "gmp.c"
 void
 gcl_init_big1(void) {
     mp_set_memory_functions( gcl_gmp_alloc,gcl_gmp_realloc,gcl_gmp_free);
+    jmp_gmp=0;
+
+#if __GNU_MP_VERSION > 4 || (__GNU_MP_VERSION == 4 && __GNU_MP_VERSION_MINOR >= 2)
+    Mersenne_Twister_Generator_Noseed.b=__gmp_randget_mt;
+    Mersenne_Twister_Generator_Noseed.c=__gmp_randclear_mt;
+    Mersenne_Twister_Generator_Noseed.d=__gmp_randiset_mt;
+#endif
+
 }
 
 #else
@@ -118,37 +130,43 @@ new_bignum(void)
    it, and then we can copy it back
 */   
 #define GCPROTECT(u) \
- MP_INT * __u = MP(big_gcprotect); \
+ MP_INT * __u = (u==MP(big_fixnum1) || u==MP(big_fixnum2) || u==MP(big_fixnum3) || u==MP(big_fixnum4) || u==MP(big_fixnum5)) ? u : MP(big_gcprotect);      \
  (__u)->_mp_d =   (u)->_mp_d; \
  (__u)->_mp_alloc = (u)->_mp_alloc 
 #define GC_PROTECTED_SELF (__u)->_mp_d
 #define END_GCPROTECT (__u)->_mp_d = 0
  
-
-
 static object
-make_bignum(__mpz_struct *u)
-{ object ans ;
- int size;
- {BEGIN_NO_INTERRUPT;
- /* make sure we follow the bignum body of u if it gets moved... */
- { GCPROTECT(u);
- ans = alloc_object(t_bignum);
- size = u->_mp_size;
- MP(ans)->_mp_d = 0;
- if (size == 0 )
-   size = 1;
- else if (size < 0) size= -size;
- MP(ans)->_mp_d = (mp_ptr) gcl_gmp_alloc (size*MP_LIMB_SIZE);
- MP(ans)->_mp_alloc = size;
- MP(ans)->_mp_size = u->_mp_size;
- memcpy(MP(ans)->_mp_d,GC_PROTECTED_SELF,size*MP_LIMB_SIZE);
- END_GCPROTECT;
- }
- END_NO_INTERRUPT;
- return ans;
- }
-} 
+make_bignum(__mpz_struct *u) {
+  object ans=alloc_object(t_bignum);
+  memset(MP(ans),0,sizeof(*MP(ans)));
+  mpz_init_set(MP(ans),u);
+  return ans;
+}
+
+/* static object */
+/* make_bignum(__mpz_struct *u) */
+/* { object ans ; */
+/*  int size; */
+/*  {BEGIN_NO_INTERRUPT; */
+/*  /\* make sure we follow the bignum body of u if it gets moved... *\/ */
+/*  { GCPROTECT(u); */
+/*  ans = alloc_object(t_bignum); */
+/*  size = u->_mp_size; */
+/*  MP(ans)->_mp_d = 0; */
+/*  if (size == 0 ) */
+/*    size = 1; */
+/*  else if (size < 0) size= -size; */
+/*  MP(ans)->_mp_d = (mp_ptr) gcl_gmp_alloc (size*MP_LIMB_SIZE); */
+/*  MP(ans)->_mp_alloc = size; */
+/*  MP(ans)->_mp_size = u->_mp_size; */
+/*  memcpy(MP(ans)->_mp_d,GC_PROTECTED_SELF,size*MP_LIMB_SIZE); */
+/*  END_GCPROTECT; */
+/*  } */
+/*  END_NO_INTERRUPT; */
+/*  return ans; */
+/*  } */
+/* }  */
 
 /* coerce a mpz_t to a bignum or fixnum */
 
@@ -156,7 +174,7 @@ object
 make_integer(__mpz_struct *u)
 {
   if ((u)->_mp_size == 0) return small_fixnum(0);
-  if (mpz_fits_sint_p(u)) {
+  if (mpz_fits_slong_p(u)) {
     return make_fixnum(mpz_get_si(u));
       }
   return make_bignum(u);
@@ -172,8 +190,8 @@ make_integer_clear(u)
 mpz_t u;
 { object ans;
   if ((u)->_mp_size == 0) return small_fixnum(0);
-  if (mpz_fits_sint_p(u)) {
-    signed long int x = mpz_get_si(u);
+  if (mpz_fits_slong_p(u)) {
+    fixnum x = mpz_get_si(u);
     mpz_clear(u);
     return make_fixnum(x);
       }
@@ -273,10 +291,9 @@ object
 normalize_big(object x)
 {
  if (MP_SIZE(x) == 0) return small_fixnum(0);
-  if (mpz_fits_sint_p(MP(x))) {
+  if (mpz_fits_slong_p(MP(x))) {
     MP_INT *u = MP(x);
-    signed long int xx = mpz_get_si(u);
-    return make_fixnum(xx);
+    return make_fixnum(mpz_get_si(u));
       }
   else return x;
 }
@@ -302,13 +319,30 @@ big_minus(object x)
 /* } */
 
 	
-double
-big_to_double(object x)
-{
-  return mpz_get_d(MP(x));
-}
+#ifndef IEEEFLOAT
+#error big_to_double requires IEEEFLOAT
+#endif
 
 
+double
+big_to_double(object x) {
+
+  int s=mpz_sizeinbase(MP(x),2),i=0,j;
+
+  if (s>=54 && mpz_tstbit(MP(x),s-54))
+    for (i=mpz_tstbit(MP(x),s-53),j=s-55;!i && j>=0 && !(i=mpz_tstbit(MP(x),j));j--);
+
+  if (i) {
+    mpz_set_si(MP(big_fixnum1),mpz_sgn(MP(x))>0 ? 1 : -1);
+    mpz_mul_2exp(MP(big_fixnum1),MP(big_fixnum1),s-54);
+    mpz_add(MP(big_fixnum1),MP(big_fixnum1),MP(x));
+    x=big_fixnum1;
+  }
+
+  return mpz_get_d(MP(x));
+
+}
+
 /* static object copy_big(object x) */
 /* { */
 /*   if (type_of(x)==t_bignum) */
@@ -420,6 +454,13 @@ otoi(object x) {
   FEwrong_type_argument(sLinteger,x);
   return NULL;
 }
+
+MP_INT *
+stoi(fixnum x) {
+  object y = new_bignum();
+  mpz_set_si(MP(y),x);
+  return MP(y);
+}
 /* end added section for declare integer -- CM */
 
 
@@ -428,25 +469,33 @@ otoi(object x) {
 /* return object like *xpt coercing to a fixnum if necessary,
    or return the actual bignum replacing it with another
 */
+
 object
-maybe_replace_big(object x)
-{ 
-/* note  mpz_fits_sint_p(MP(x)) returns arbitrary result if
-   passed 0 in bignum form.
-   bug or feature of gmp..
-*/   
-  if (MP_SIZE(x) == 0) return small_fixnum(0);
-  if (mpz_fits_sint_p(MP(x))) {
-    MP_INT *u = MP(x);
-    signed long int xx = mpz_get_si(u);
-    return make_fixnum(xx);
-  }
+replace_big(object x) {
+
   return make_bignum(MP(x));
+
+}
+
+object
+maybe_replace_big(object x) {
+
+  /* note  mpz_fits_sint_p(MP(x)) returns arbitrary result if
+     passed 0 in bignum form.
+     bug or feature of gmp..
+  */
+  if (MP_SIZE(x) == 0) return small_fixnum(0);
+  if (mpz_fits_slong_p(MP(x))) {
+    MP_INT *u = MP(x);
+    return make_fixnum(mpz_get_si(u));
+  }
+  return replace_big(x);
+
 }
 
 
 object
-bignum2(unsigned int h, unsigned int l)
+bignum2( unsigned int h,  unsigned int l)
 {
   object x = new_bignum();
   mpz_set_ui(MP(x),h);
@@ -456,53 +505,88 @@ bignum2(unsigned int h, unsigned int l)
 }
 
 void
-integer_quotient_remainder_1(object x, object y, object *qp, object *rp)
-{
-  *qp = new_bignum();
-  *rp = new_bignum();
-  /* we may need to coerce the fixnums to MP here, and
-     we use the temporary storage of the rp/qp as inputs.
-     since overlap is allowed in the mpz_tdiv_qr operation..
-  */    
-  mpz_tdiv_qr(MP(*qp),MP(*rp),INTEGER_TO_MP(x,big_fixnum1),
-	      INTEGER_TO_MP(y,big_fixnum2));
-  *qp = normalize_big(*qp);
-  *rp = normalize_big(*rp);
-  return;
+integer_quotient_remainder_1(object x, object y, object *qp, object *rp,fixnum d) {
+
+  if (type_of(x)==t_fixnum && type_of(y)==t_fixnum) {
+    fixnum fx=fix(x),fy=fix(y);
+    if (fx!=MOST_NEGATIVE_FIX) {
+      if (qp) {
+	fixnum z=fixnum_div(fx,fy,d);
+	if (rp) *rp=make_fixnum(fx-fy*z);
+	*qp=make_fixnum(z);
+      } else if (rp)
+	*rp=make_fixnum(fixnum_rem(fx,fy,d));
+      return;
+    }
+  }
+
+  {
+
+    __mpz_struct *b1=INTEGER_TO_MP(x,big_fixnum1),*b2=INTEGER_TO_MP(y,big_fixnum2);
+
+    if (qp) {
+      if (rp) {
+	void (*f)()=d<0 ? mpz_fdiv_qr : (d>0 ? mpz_cdiv_qr : mpz_tdiv_qr);
+	f(MP(big_fixnum3),MP(big_fixnum4),b1,b2);
+	*rp=maybe_replace_big(big_fixnum4);
+      } else {
+	void (*f)()=d<0 ? mpz_fdiv_q : (d>0 ? mpz_cdiv_q : mpz_tdiv_q);
+	f(MP(big_fixnum3),b1,b2);
+      }
+      *qp=maybe_replace_big(big_fixnum3);
+    } else if (rp) {
+      void (*f)()=d<0 ? mpz_fdiv_r : (d>0 ? mpz_cdiv_r : mpz_tdiv_r);
+      f(MP(big_fixnum4),b1,b2);
+      *rp=maybe_replace_big(big_fixnum4);
+    }
+
+  }
+
 }
 
+
 #define HAVE_MP_COERCE_TO_STRING
      
 object
-coerce_big_to_string(object x, int printbase)
-{ int i;
- int sign = BIG_SIGN(x);
- int ss = mpz_sizeinbase(MP(x),printbase);
- char *p;
-  object ans = alloc_simple_string(ss+2+(sign<0? 1: 0));
-  ans->st.st_self=p=alloc_relblock(ans->st.st_dim);
-  /*  if (sign < 0) *p++='-'; */
-  mpz_get_str(p, printbase,MP(x));
-  i = ans->st.st_dim-5;
-  if (i <0 ) i=0;
-  while(ans->st.st_self[i]) { i++;}
-  ans->st.st_fillp=i;
+coerce_big_to_string(object x, int printbase) {
+
+  int ss=mpz_sizeinbase(MP(x),printbase)+(BIG_SIGN(x)<0 ? 1 : 0)+1;
+  char *p;
+  object ans;
+
+  massert(p=alloca(ss));
+  massert(p=mpz_get_str(p,printbase,MP(x)));
+  ss=strlen(p);
+  ans=alloc_simple_string(ss);
+  ans->st.st_self=alloc_relblock(ss);
+  memcpy(ans->st.st_self,p,ss);
+
   return ans;
+
 }
 
 
 void
 gcl_init_big(void)
 {
-  big_gcprotect=alloc_object(t_bignum);
+  gcl_init_big1();
+  big_gcprotect=alloc_object(t_bignum);/*FIXME*/
   MP_SELF(big_gcprotect)=0;
   MP_ALLOCATED(big_gcprotect)=0;
-  big_fixnum1=new_bignum();
-  big_fixnum2=new_bignum();
-  enter_mark_origin(&big_fixnum1);
   enter_mark_origin(&big_gcprotect);
-  enter_mark_origin(&big_fixnum2);
-  gcl_init_big1();
 
+#define mjoin(a_,b_) a_ ## b_
+#define Mjoin(a_,b_) mjoin(a_,b_)
+#define init_big_fixnum(a_) {			\
+    Mjoin(big_fixnum,a_)=new_bignum();		\
+    mpz_set_si(MP(Mjoin(big_fixnum,a_)),0);	\
+    enter_mark_origin(&Mjoin(big_fixnum,a_));	\
+  }
+  init_big_fixnum(1);
+  init_big_fixnum(2);
+  init_big_fixnum(3);
+  init_big_fixnum(4);
+  init_big_fixnum(5);
+  sSPminus_most_negative_fixnumP=make_si_constant("+MINUS-MOST-NEGATIVE-FIXNUM+",fixnum_add(MOST_POSITIVE_FIX,1));
 
 }
