@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -174,8 +175,7 @@ number_evenp(object x)
 	return(~i & 1);
 }
 
-void
-Lzerop(void)
+LFD(Lzerop)(void)
 {
 	check_arg(1);
 	check_type_number(&vs_base[0]);
@@ -185,8 +185,7 @@ Lzerop(void)
 		vs_base[0] = Cnil;
 }
 
-void
-Lplusp(void)
+LFD(Lplusp)(void)
 {
 	check_arg(1);
 	check_type_or_rational_float(&vs_base[0]);
@@ -196,8 +195,7 @@ Lplusp(void)
 		vs_base[0] = Cnil;
 }
 
-void
-Lminusp(void)
+LFD(Lminusp)(void)
 {
 	check_arg(1);
 	check_type_or_rational_float(&vs_base[0]);
@@ -207,8 +205,7 @@ Lminusp(void)
 		vs_base[0] = Cnil;
 }
 
-void
-Loddp(void)
+LFD(Loddp)(void)
 {
 	check_arg(1);
 	check_type_integer(&vs_base[0]);
@@ -218,8 +215,7 @@ Loddp(void)
 		vs_base[0] = Cnil;
 }
 
-void
-Levenp(void)
+LFD(Levenp)(void)
 {
 	check_arg(1);
 	check_type_integer(&vs_base[0]);
@@ -243,7 +239,7 @@ Levenp(void)
 
 
 void
-init_num_pred(void)
+gcl_init_num_pred(void)
 {
 #ifndef GMP
         big_register_1 = new_bignum();
