@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -68,7 +69,7 @@ rando(object x, object rs) {
     d=mpz_get_d(MP(out));
     d/=(double)fbase;
     z=alloc_object(tx);
-    if (tx==t_shortfloat) sf(z)=sf(x)*d; else lf(z)=lf(x)*d;
+    BLOCK_EXCEPTIONS(if (tx==t_shortfloat) sf(z)=sf(x)*d; else lf(z)=lf(x)*d);
     return z;
   default:
     FEerror("~S is not an integer nor a floating-point number.", 1, x);
@@ -104,6 +105,17 @@ trap_gcl_gmp_allocfun(size_t size){
 }
 #endif
 
+void
+reinit_gmp() {
+
+#if __GNU_MP_VERSION > 4 || (__GNU_MP_VERSION == 4 && __GNU_MP_VERSION_MINOR >= 2)
+  Mersenne_Twister_Generator_Noseed.b=__gmp_randget_mt;
+  Mersenne_Twister_Generator_Noseed.c=__gmp_randclear_mt;
+  Mersenne_Twister_Generator_Noseed.d=__gmp_randiset_mt;
+#endif
+
+}
+
 void
 init_gmp_rnd_state(__gmp_randstate_struct *x) {
 
