@@ -1,5 +1,6 @@
 /* 
 Copyright William Schelter. All rights reserved.
+Copyright 2024 Camm Maguire
 There is a companion file rsym.c which is used to build
 a list of the external symbols in a COFF or A.OUT object file, for
 example saved_kcl.  These are loaded into kcl, and the
@@ -60,16 +61,16 @@ find_sym_ptable(const char *name) {
 
 }
 
-DEFUN_NEW("FIND-SYM-PTABLE",object,fSfind_sym_ptable,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
+DEFUN("FIND-SYM-PTABLE",object,fSfind_sym_ptable,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
   char c;
   struct node *a;
 
   check_type_string(&x);
 
-  c=x->st.st_self[x->st.st_fillp];
-  x->st.st_self[x->st.st_fillp]=0;
+  c=x->st.st_self[VLEN(x)];
+  x->st.st_self[VLEN(x)]=0;
   a=find_sym_ptable(x->st.st_self);
-  x->st.st_self[x->st.st_fillp]=c;
+  x->st.st_self[VLEN(x)]=c;
 
   return (object)(a ? a->address : 0);
 
