@@ -1,4 +1,5 @@
 /*  Copyright William Schelter. All rights reserved.
+    Copyright 2024 Camm Maguire
     
     Stratified Garbage Collection  (SGC)
     
@@ -672,7 +673,7 @@ sgc_start(void) {
 
   }
 
-  sSAwritableA->s.s_dbind=fSmake_vector1_1((page(heap_end)-first_data_page),aet_bit,Ct);
+  sSAwritableA->s.s_dbind=fSmake_vector(sLbit,(page(heap_end)-first_data_page),Ct,Cnil,Cnil,0,Ct,Cnil);
   wrimap=(void *)sSAwritableA->s.s_dbind->v.v_self;
 
   /* now move the sgc free lists into place.   alt_free should
@@ -774,8 +775,8 @@ sgc_start(void) {
       object x;
 
       for (x=malloc_list;x!=Cnil;x=x->c.c_cdr)
-	if (x->c.c_car->st.st_adjustable)
-	  for (i=page(x->c.c_car->st.st_self);i<=page(x->c.c_car->st.st_self+x->c.c_car->st.st_fillp-1);i++)
+	if (x->c.c_car->st.st_writable)
+	  for (i=page(x->c.c_car->st.st_self);i<=page(x->c.c_car->st.st_self+VLEN(x->c.c_car)-1);i++)
 	    SET_WRITABLE(i);
     }
 
@@ -911,7 +912,7 @@ sgc_quit(void) {
 
 fixnum debug_fault =0;
 fixnum fault_count =0;
-extern char etext;
+
 static void
 memprotect_handler(int sig, long code, void *scp, char *addr) {
   
@@ -962,7 +963,7 @@ memprotect_handler(int sig, long code, void *scp, char *addr) {
   INSTALL_MPROTECT_HANDLER;
 #endif
 
-  segmentation_catcher(0);
+  segmentation_catcher(sig,code,scp,addr);
 
 }
 
