@@ -1,5 +1,6 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
+ Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -25,7 +26,6 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 	structure interface
 */
 
-#include <string.h>
 #include "include.h"
 
 
@@ -54,20 +54,13 @@ bad_raw_type(void)
 {     	  FEerror("Bad raw struct type",0);}
 
 
-DEFUN_NEW("STRUCTURE-DEF",object,fSstructure_def,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+DEFUN("STRUCTURE-DEF",object,fSstructure_def,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
   check_type_structure(x);
   return (x)->str.str_def;
 }
 
-DEFUN_NEW("STRUCTURE-LENGTH",object,fSstructure_length,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
-  check_type_structure(x);
-  return (object)S_DATA(x)->length;
-}
+DEFUN("STRUCTURE-REF",object,structure_ref,SI,3,3,NONE,OO,OI,OO,OO,(object x,object name,fixnum i),"") {
 
-DEFUN_NEW("STRUCTURE-REF",object,structure_ref,SI,3,3,NONE,OO,OI,OO,OO,(object x,object name,fixnum i),"") {
-/* object */
-/* structure_ref(object x, object name, int i) */
-/* { */
   unsigned short *s_pos;
   COERCE_DEF(name);
   if (type_of(x) != t_structure ||
@@ -80,15 +73,17 @@ DEFUN_NEW("STRUCTURE-REF",object,structure_ref,SI,3,3,NONE,OO,OI,OO,OO,(object x
   switch((SLOT_TYPE(x->str.str_def,i)))
     {
     case aet_object: return(STREF(object,x,s_pos[i]));
-    case aet_fix:  return(make_fixnum((STREF(fixnum,x,s_pos[i]))));
+    case aet_nnfix: case aet_fix:  return(make_fixnum((STREF(fixnum,x,s_pos[i]))));
     case aet_ch:  return(code_char(STREF(char,x,s_pos[i])));
     case aet_bit:
-    case aet_char: return(small_fixnum(STREF(char,x,s_pos[i])));
+    case aet_nnchar: case aet_char: return(small_fixnum(STREF(char,x,s_pos[i])));
     case aet_sf: return(make_shortfloat(STREF(shortfloat,x,s_pos[i])));
     case aet_lf: return(make_longfloat(STREF(longfloat,x,s_pos[i])));
     case aet_uchar: return(small_fixnum(STREF(unsigned char,x,s_pos[i])));
     case aet_ushort: return(make_fixnum(STREF(unsigned short,x,s_pos[i])));
-    case aet_short: return(make_fixnum(STREF(short,x,s_pos[i])));
+    case aet_nnshort: case aet_short: return(make_fixnum(STREF(short,x,s_pos[i])));
+    case aet_uint: return(make_fixnum(STREF(unsigned int,x,s_pos[i])));
+    case aet_nnint: case aet_int: return(make_fixnum(STREF(int,x,s_pos[i])));
     default:
       bad_raw_type();
       return 0;
@@ -115,12 +110,10 @@ FFN(siLstructure_ref1)(void)
  vs_top=vs_base+1;
 }
 
-DEFUN_NEW("STRUCTURE-SET",object,structure_set,SI,4,4,NONE,OO,OI,OO,OO,(object x,object name,fixnum i,object v),"") {
-/* object */
-/* structure_set(object x, object name, int i, object v) */
-/* { */
+DEFUN("STRUCTURE-SET",object,structure_set,SI,4,4,NONE,OO,OI,OO,OO,(object x,object name,fixnum i,object v),"") {
+
   unsigned short *s_pos;
-  
+
   COERCE_DEF(name);
   if (type_of(x) != t_structure ||
       type_of(name) != t_structure ||
@@ -128,30 +121,32 @@ DEFUN_NEW("STRUCTURE-SET",object,structure_set,SI,4,4,NONE,OO,OI,OO,OO,(object x
     FEwrong_type_argument((type_of(name)==t_structure ?
 			   S_DATA(name)->name : name)
 			  , x);
-  
+
 #ifdef SGC
   /* make sure the structure header is on a writable page */
   if (is_marked(x)) FEerror("bad gc field",0); else  unmark(x);
 #endif   
-  
-  s_pos= & SLOT_POS(x->str.str_def,0);
-  switch(SLOT_TYPE(x->str.str_def,i)){
-    
-  case aet_object: STREF(object,x,s_pos[i])=v; break;
-  case aet_fix:  (STREF(fixnum,x,s_pos[i]))=fix(v); break;
-  case aet_ch:  STREF(char,x,s_pos[i])=char_code(v); break;
-  case aet_bit:
-  case aet_char: STREF(char,x,s_pos[i])=fix(v); break;
-  case aet_sf: STREF(shortfloat,x,s_pos[i])=sf(v); break;
-  case aet_lf: STREF(longfloat,x,s_pos[i])=lf(v); break;
-  case aet_uchar: STREF(unsigned char,x,s_pos[i])=fix(v); break;
-  case aet_ushort: STREF(unsigned short,x,s_pos[i])=fix(v); break;
-  case aet_short: STREF(short,x,s_pos[i])=fix(v); break;
-  default:
-    bad_raw_type();
-    
-  }
-  return(v);
+ 
+ s_pos= & SLOT_POS(x->str.str_def,0);
+ switch(SLOT_TYPE(x->str.str_def,i)){
+   
+   case aet_object: STREF(object,x,s_pos[i])=v; break;
+   case aet_nnfix:case aet_fix:  (STREF(fixnum,x,s_pos[i]))=fix(v); break;
+   case aet_ch:  STREF(char,x,s_pos[i])=char_code(v); break;
+   case aet_bit:
+   case aet_nnchar:case aet_char: STREF(char,x,s_pos[i])=fix(v); break;
+   case aet_sf: STREF(shortfloat,x,s_pos[i])=sf(v); break;
+   case aet_lf: STREF(longfloat,x,s_pos[i])=lf(v); break;
+   case aet_uchar: STREF(unsigned char,x,s_pos[i])=fix(v); break;
+   case aet_ushort: STREF(unsigned short,x,s_pos[i])=fix(v); break;
+   case aet_nnshort:case aet_short: STREF(short,x,s_pos[i])=fix(v); break;
+   case aet_uint: STREF(unsigned int,x,s_pos[i])=fix(v); break;
+   case aet_nnint:case aet_int: STREF(int,x,s_pos[i])=fix(v); break;
+ default:
+   bad_raw_type();
+
+   }
+ return(v);
 }
 #ifdef STATIC_FUNCTION_POINTERS
 object
@@ -160,20 +155,25 @@ structure_set(object x,object name,fixnum i,object v) {
 }
 #endif
 
-static void
-FFN(siLstructure_subtype_p)(void)
-{object x,y;
- check_arg(2);
- x=vs_base[0];
- y=vs_base[1];
- if (type_of(x)!=t_structure)
-   {vs_base[0]=Cnil; goto BOTTOM;}
- x=x->str.str_def;
- COERCE_DEF(y);
- if (structure_subtypep(x,y)) vs_base[0]=Ct;
- else vs_base[0]=Cnil;
- BOTTOM:
- vs_top=vs_base+1;
+DEFUN("STRUCTURE-LENGTH",object,fSstructure_length,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
+  check_type_structure(x);
+  return (object)S_DATA(x)->length;
+}
+
+DEFUN("STRUCTURE-SUBTYPE-P",object,fSstructure_subtype_p,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+/* static void */
+/* FFN(siLstructure_subtype_p)(void) */
+/* {object x,y; */
+/*  check_arg(2); */
+/*  x=vs_base[0]; */
+/*  y=vs_base[1]; */
+  if (type_of(x)!=t_structure)
+    RETURN1(Cnil);
+  x=x->str.str_def;
+  COERCE_DEF(y);
+  RETURN1(structure_subtypep(x,y) ? Ct : Cnil);
+
 }
  
      
@@ -188,7 +188,7 @@ structure_to_list(object x)
 
   s=def->slot_descriptions;
   for (p=&v,i=0,n=def->length;!endp(s)&&i<n;s=s->c.c_cdr,i++) {
-    collect(p,make_cons(car(s->c.c_car),Cnil));
+    collect(p,make_cons(intern(car(s->c.c_car),keyword_package),Cnil));
     collect(p,make_cons(structure_ref(x,x->str.str_def,i),Cnil));
   }
   *p=Cnil;
@@ -197,80 +197,98 @@ structure_to_list(object x)
 
 }
 
-LFD(siLmake_structure)(void)
-{
-  object x,name,*base;
-  struct s_data *def=NULL;
-  int narg, i,size;
-  base=vs_base;
-  if ((narg = vs_top - base) == 0)
-    too_few_arguments();
-  {BEGIN_NO_INTERRUPT;
+DEFUN("MAKE-DUMMY-STRUCTURE",object,fSmake_dummy_structure,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+  
+  object x;
+
+  BEGIN_NO_INTERRUPT;
   x = alloc_object(t_structure);
-  name=base[0];
-  COERCE_DEF(name);
-  if (type_of(name)!=t_structure ||
-      (def=S_DATA(name))->length != --narg)
-    FEerror("Bad make_structure args for type ~a",1,base[0]);
-  x->str.str_def = name;
-  x->str.str_self = NULL;
-  size=S_DATA(name)->size;
-  base[0] = x;
-  x->str.str_self = (object *)
-    (def->staticp == Cnil ? alloc_relblock(size)
-     : alloc_contblock(size));
+  x->str.str_def=NULL;
+  x->str.str_self=NULL;
+  END_NO_INTERRUPT;
+  return x;
+}
+
+DEFUN("MAKE-STRUCTURE",object,fSmake_structure,SI,1,63,NONE,OO,OO,OO,OO,(object name,...),"") {/*FIXME*/
+
+  fixnum narg=INIT_NARGS(1),i,size;
+  object l=Cnil,f=OBJNULL,v,x;
+  struct s_data *def=NULL;
+  va_list ap;
+  unsigned char *s_type;
+  unsigned short *s_pos;
+
+  {
+    BEGIN_NO_INTERRUPT;
+    x = alloc_object(t_structure);
+    COERCE_DEF(name);
+    if (type_of(name)!=t_structure  ||
+	(def=S_DATA(name))->length != narg)
+      FEerror("Bad make_structure args for type ~a",1,name);
+    x->str.str_def = name;
+    x->str.str_self = NULL;
+    size=S_DATA(name)->size;
+    x->str.str_self=(object *)(def->staticp == Cnil ? alloc_relblock(size) : alloc_contblock(size));
   /* There may be holes in the structure.
      We want them zero, so that equal can work better.
      */
-  if (S_DATA(name)->has_holes != Cnil)
-    bzero(x->str.str_self,size);
-  {unsigned char *s_type;
-   unsigned short *s_pos;
-   s_pos= (&SLOT_POS(x->str.str_def,0));
-   s_type = (&(SLOT_TYPE(x->str.str_def,0)));
-   base=base+1;
-   for (i = 0;  i < narg;  i++)
-     {object v=base[i];
-      switch(s_type[i]){
+    if (S_DATA(name)->has_holes != Cnil)
+      bzero(x->str.str_self,size);
+
+    s_pos= (&SLOT_POS(x->str.str_def,0));
+    s_type = (&(SLOT_TYPE(x->str.str_def,0)));
+
+    va_start(ap,name);
+    for (i=0;(v=NEXT_ARG(narg,ap,l,f,OBJNULL))!=OBJNULL;i++) {
+
+      switch(s_type[i]) {
 	     
       case aet_object: STREF(object,x,s_pos[i])=v; break;
-      case aet_fix:  (STREF(fixnum,x,s_pos[i]))=fix(v); break;
+      case aet_nnfix:case aet_fix:  (STREF(fixnum,x,s_pos[i]))=fix(v); break;
       case aet_ch:  STREF(char,x,s_pos[i])=char_code(v); break;
       case aet_bit:
-      case aet_char: STREF(char,x,s_pos[i])=fix(v); break;
+      case aet_nnchar:case aet_char: STREF(char,x,s_pos[i])=fix(v); break;
       case aet_sf: STREF(shortfloat,x,s_pos[i])=sf(v); break;
       case aet_lf: STREF(longfloat,x,s_pos[i])=lf(v); break;
       case aet_uchar: STREF(unsigned char,x,s_pos[i])=fix(v); break;
       case aet_ushort: STREF(unsigned short,x,s_pos[i])=fix(v); break;
-      case aet_short: STREF(short,x,s_pos[i])=fix(v); break;
+      case aet_nnshort:case aet_short: STREF(short,x,s_pos[i])=fix(v); break;
+	/*FIXME uint on 32bit really should not be here*/
+      case aet_uint: STREF(unsigned int,x,s_pos[i])=fix(v); break;
+      case aet_nnint:case aet_int: STREF(int,x,s_pos[i])=fix(v); break;
       default:
 	bad_raw_type();
 
-      }}
-   vs_top = base;
-   vs_base=base-1;
-  END_NO_INTERRUPT;}
- }
+      }
+    }
+
+    va_end(ap);
+    END_NO_INTERRUPT;
+
+  }
+
+  RETURN1(x);
+
 }
 
-static void
-FFN(Lcopy_structure)(void)
-{
-	object x, y;
-	struct s_data *def;
+DEFUN("COPY-STRUCTURE",object,fLcopy_structure,LISP,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+
+  object y;
+  struct s_data *def;
+  
+  check_type_structure(x);
+  {
+    BEGIN_NO_INTERRUPT;
+    y = alloc_object(t_structure);
+    def=S_DATA(y->str.str_def = x->str.str_def);
+    y->str.str_self = NULL;
+    y->str.str_self = (object *)alloc_relblock(def->size);
+    memcpy(y->str.str_self,x->str.str_self,def->size);
+    END_NO_INTERRUPT;
+  }
+
+  return y;
 
-	check_arg(1);
-/*  	if (vs_top-vs_base < 1) too_few_arguments(); */
-	x = vs_base[0];
-	check_type_structure(x);
-	{BEGIN_NO_INTERRUPT;
-	vs_base[0] = y = alloc_object(t_structure);
-	def=S_DATA(y->str.str_def = x->str.str_def);
-	y->str.str_self = NULL;
-	y->str.str_self = (object *)alloc_relblock(def->size);
-	bcopy(x->str.str_self,y->str.str_self,def->size);
-	vs_top=vs_base+1;
-        END_NO_INTERRUPT;}
 }
 
 LFD(siLstructure_name)(void)
@@ -297,65 +315,65 @@ LFD(siLstructure_set)(void)
 LFD(siLstructurep)(void)
 {
 	check_arg(1);
-	if (type_of(vs_base[0]) == t_structure)
+	if (type_of(vs_base[0]) == t_structure && !vs_base[0]->d.tt)
 		vs_base[0] = Ct;
 	else
 		vs_base[0] = Cnil;
 }
 
-LFD(siLrplaca_nthcdr)(void)
-{
+/* LFD(siLrplaca_nthcdr)(void) */
+/* { */
 
-/*
-	Used in DEFSETF forms generated by DEFSTRUCT.
-	(si:rplaca-nthcdr x i v) is equivalent to 
-	(progn (rplaca (nthcdr i x) v) v).
-*/
-	int i;
-	object l;
+/* /\* */
+/* 	Used in DEFSETF forms generated by DEFSTRUCT. */
+/* 	(si:rplaca-nthcdr x i v) is equivalent to  */
+/* 	(progn (rplaca (nthcdr i x) v) v). */
+/* *\/ */
+/* 	int i; */
+/* 	object l; */
 
-	check_arg(3);
-	if (type_of(vs_base[1]) != t_fixnum || fix(vs_base[1]) < 0)
-		FEerror("~S is not a non-negative fixnum.", 1, vs_base[1]);
-	if (type_of(vs_base[0]) != t_cons)
-		FEerror("~S is not a cons.", 1, vs_base[0]);
+/* 	check_arg(3); */
+/* 	if (type_of(vs_base[1]) != t_fixnum || fix(vs_base[1]) < 0) */
+/* 		FEerror("~S is not a non-negative fixnum.", 1, vs_base[1]); */
+/* 	if (!consp(vs_base[0])) */
+/* 		FEerror("~S is not a cons.", 1, vs_base[0]); */
 
-	for (i = fix(vs_base[1]), l = vs_base[0];  i > 0; --i) {
-		l = l->c.c_cdr;
-		if (endp(l))
-			FEerror("The offset ~S is too big.", 1, vs_base[1]);
-	}
-	take_care(vs_base[2]);
-	l->c.c_car = vs_base[2];
-	vs_base = vs_base + 2;
-}
+/* 	for (i = fix(vs_base[1]), l = vs_base[0];  i > 0; --i) { */
+/* 		l = l->c.c_cdr; */
+/* 		if (endp(l)) */
+/* 			FEerror("The offset ~S is too big.", 1, vs_base[1]); */
+/* 	} */
+/* 	take_care(vs_base[2]); */
+/* 	l->c.c_car = vs_base[2]; */
+/* 	vs_base = vs_base + 2; */
+/* } */
 
-LFD(siLlist_nth)(void)
-{
+/* LFD(siLlist_nth)(void) */
+/* { */
 
-/*
-	Used in structure access functions generated by DEFSTRUCT.
-	si:list-nth is similar to nth except that
-	(si:list-nth i x) is error if the length of the list x is less than i.
-*/
-	int i;
-	object l;
+/* /\* */
+/* 	Used in structure access functions generated by DEFSTRUCT. */
+/* 	si:list-nth is similar to nth except that */
+/* 	(si:list-nth i x) is error if the length of the list x is less than i. */
+/* *\/ */
+/* 	int i; */
+/* 	object l; */
 
-	check_arg(2);
-	if (type_of(vs_base[0]) != t_fixnum || fix(vs_base[0]) < 0)
-		FEerror("~S is not a non-negative fixnum.", 1, vs_base[0]);
-	if (type_of(vs_base[1]) != t_cons)
-		FEerror("~S is not a cons.", 1, vs_base[1]);
+/* 	check_arg(2); */
+/* 	if (type_of(vs_base[0]) != t_fixnum || fix(vs_base[0]) < 0) */
+/* 		FEerror("~S is not a non-negative fixnum.", 1, vs_base[0]); */
+/* 	if (!consp(vs_base[1])) */
+/* 		FEerror("~S is not a cons.", 1, vs_base[1]); */
 
-	for (i = fix(vs_base[0]), l = vs_base[1];  i > 0; --i) {
-		l = l->c.c_cdr;
-		if (endp(l))
-			FEerror("The offset ~S is too big.", 1, vs_base[0]);
-	}
+/* 	for (i = fix(vs_base[0]), l = vs_base[1];  i > 0; --i) { */
+/* 		l = l->c.c_cdr; */
+/* 		if (endp(l)) */
+/* 			FEerror("The offset ~S is too big.", 1, vs_base[0]); */
+/* 	} */
 
-	vs_base[0] = l->c.c_car;
-	vs_popp;
-}
+/* 	vs_base[0] = l->c.c_car; */
+/* 	vs_popp; */
+/* } */
 
 
 static void
@@ -368,7 +386,7 @@ FFN(siLmake_s_data_structure)(void)
  raw=vs_base[1];
  y=alloc_object(t_structure);
  y->str.str_def=y;
- y->str.str_self = (object *)( x->v.v_self);
+ y->str.str_self = (object *)(x->v.v_self);
  S_DATA(y)->name  =sSs_data;
  S_DATA(y)->length=(raw->v.v_dim);
  S_DATA(y)->raw   =raw;
@@ -382,35 +400,13 @@ FFN(siLmake_s_data_structure)(void)
  vs_top=vs_base+1;
 }
 
-/* static void */
-/* FFN(siLstructure_def)(void) */
-/* {check_arg(1); */
-/*  check_type_structure(vs_base[0]); */
-/*   vs_base[0]=vs_base[0]->str.str_def; */
-/* } */
-
-short aet_sizes [] = {
-sizeof(object),  /* aet_object  t  */
-sizeof(char),  /* aet_ch  string-char  */
-sizeof(char),  /* aet_bit  bit  */
-sizeof(fixnum),  /* aet_fix  fixnum  */
-sizeof(float),  /* aet_sf  short-float  */
-sizeof(double),  /* aet_lf  long-float  */
-sizeof(char),  /* aet_char  signed char */
-sizeof(char),  /* aet_uchar  unsigned char */
-sizeof(short),  /* aet_short  signed short */
-sizeof(short)  /* aet_ushort  unsigned short   */
-};
-
-  
-
-
+extern aet_type_struct aet_types[];
 
 static void
 FFN(siLsize_of)(void)
 { object x= vs_base[0];
   int i;
-  i= aet_sizes[fix(fSget_aelttype(x))];
+  i= aet_types[fix(fSget_aelttype(x))].size;
   vs_base[0]=make_fixnum(i);
 }
   
@@ -447,19 +443,16 @@ gcl_init_structure_function(void)
 {
 
 
-	make_si_function("MAKE-STRUCTURE", siLmake_structure);
+/* 	make_si_function("MAKE-STRUCTURE", siLmake_structure); */
 	make_si_function("MAKE-S-DATA-STRUCTURE",siLmake_s_data_structure);
-	make_function("COPY-STRUCTURE", Lcopy_structure);
+/* 	make_si_function("COPY-STRUCTURE", siLcopy_structure); */
 	make_si_function("STRUCTURE-NAME", siLstructure_name);
-	/* make_si_function("STRUCTURE-REF", siLstructure_ref); */
-	/* make_si_function("STRUCTURE-DEF", siLstructure_def); */
 	make_si_function("STRUCTURE-REF1", siLstructure_ref1);
-	/* make_si_function("STRUCTURE-SET", siLstructure_set); */
 	make_si_function("STRUCTUREP", siLstructurep);
 	make_si_function("SIZE-OF", siLsize_of);
 	make_si_function("ALIGNMENT",siLalignment);
-	make_si_function("STRUCTURE-SUBTYPE-P",siLstructure_subtype_p);
-	make_si_function("RPLACA-NTHCDR", siLrplaca_nthcdr);
-	make_si_function("LIST-NTH", siLlist_nth);
+/* 	make_si_function("STRUCTURE-SUBTYPE-P",siLstructure_subtype_p); */
+	/* make_si_function("RPLACA-NTHCDR", siLrplaca_nthcdr); */
+	/* make_si_function("LIST-NTH", siLlist_nth); */
 	make_si_function("AET-TYPE",siLaet_type);
 }
