@@ -1,5 +1,6 @@
 /* unexec for GNU Emacs on Windows NT.
    Copyright (C) 1994 Free Software Foundation, Inc.
+   Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Emacs.
 
@@ -26,14 +27,12 @@ Boston, MA 02111-1307, USA.
 #ifndef UNIXSAVE
 #include <config.h>
 #endif
-/* #include <stdlib.h> */ 	/* _fmode */
 /* in case the include of config.h defined it */
 #undef va_start
 #include <stdio.h>
 #include <fcntl.h>
 #include <time.h>
 #include <windows.h>
-#include <string.h>   /* strrchr */
 
 #ifdef _GNU_H_WINDOWS_H 
 #include "cyglacks.h"
@@ -52,7 +51,6 @@ PIMAGE_NT_HEADERS
 
 
 
-#include <stdlib.h>
 #include <stdio.h>
 
 #include "ntheap.h"
